﻿namespace JSLintNet.VisualStudio.Errors
{
    using System;
    using System.Collections.Generic;
    using System.Text.RegularExpressions;
    using Microsoft.VisualStudio.Text;
    using Microsoft.VisualStudio.Text.Tagging;

    internal class JSLintTagger : ITagger<IErrorTag>
    {
        private static readonly Regex WordBoundaryPattern = new Regex(@"[^\$\w]", RegexOptions.Compiled);

        private ITextBuffer buffer;

        private IJSLintErrorListProvider errorListProvider;

        private string fileName;

        public JSLintTagger(ITextBuffer buffer, IJSLintErrorListProvider errorListProvider, string fileName)
        {
            this.buffer = buffer;
            this.errorListProvider = errorListProvider;
            this.fileName = fileName;

            this.errorListProvider.ErrorListChange += this.OnErrorListChange;
        }

        public event EventHandler<SnapshotSpanEventArgs> TagsChanged;

        public IEnumerable<ITagSpan<IErrorTag>> GetTags(NormalizedSnapshotSpanCollection snapshotSpans)
        {
            var errors = this.errorListProvider.GetErrors(this.fileName);
            var list = new List<TagSpan<IErrorTag>>();

            if (errors.Count > 0)
            {
                foreach (var snapshotSpan in snapshotSpans)
                {
                    var firstLine = snapshotSpan.Snapshot.GetLineNumberFromPosition(snapshotSpan.Span.Start);
                    var lastLine = snapshotSpan.Snapshot.GetLineNumberFromPosition(snapshotSpan.Span.End);

                    foreach (var error in errors)
                    {
                        if (error.Line >= firstLine && error.Line <= lastLine)
                        {
                            var errorSpan = GetErrorSpan(snapshotSpan.Snapshot, error);

                            if (errorSpan.IntersectsWith(snapshotSpan.Span))
                            {
                                var tagSpan = new TagSpan<IErrorTag>(new SnapshotSpan(snapshotSpan.Snapshot, errorSpan), error);
                                list.Add(tagSpan);
                            }
                        }
                    }
                }
            }

            return list;
        }

        private static Span GetErrorSpan(ITextSnapshot snapshot, JSLintErrorTask error)
        {
            var line = snapshot.GetLineFromLineNumber(error.Line);
            var text = line.GetText();

            if (text.Length < error.Column)
            {
                return new Span(line.End.Position, 1);
            }

            var start = line.Start.Position + error.Column;
            var length = line.End.Position - start;
            var match = WordBoundaryPattern.Match(text, error.Column);

            if (match.Success)
            {
                length = match.Index - error.Column;
            }

            return new Span(start, length);
        }

        private void OnErrorListChange(object sender, ErrorListChangeEventArgs e)
        {
            var handler = this.TagsChanged;

            if (handler != null && this.ShouldRelayChange(e))
            {
                var snapshot = this.buffer.CurrentSnapshot;
                var span = new SnapshotSpan(snapshot, new Span(0, snapshot.Length));

                handler(this, new SnapshotSpanEventArgs(span));
            }
        }

        private bool ShouldRelayChange(ErrorListChangeEventArgs e)
        {
            switch (e.Action)
            {
                case ErrorListAction.ClearFile:
                case ErrorListAction.AddFile:
                    return this.fileName == e.FileName;
                case ErrorListAction.ClearAll:
                    return true;
            }

            return false;
        }
    }
}
