﻿namespace JSLintNet
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Linq;
    using System.Text;
    using JSLintNet.Abstractions;
    using JSLintNet.Helpers;
    using JSLintNet.Json;
    using JSLintNet.Properties;
    using Microsoft.Build.Framework;
    using Microsoft.Build.Utilities;

    /// <summary>
    /// Task that will run JSLint over source files.
    /// </summary>
    public class JSLintTask : Task
    {
        private const string JSLintSubcategory = "JSLint";

        private IJSLintFactory jsLintFactory;

        private IFileSystemWrapper fileSystemWrapper;

        private IJsonProvider jsonProvider;

        /// <summary>
        /// Initializes a new instance of the <see cref="JSLintTask"/> class with default services.
        /// </summary>
        public JSLintTask()
            : this(new JSLintFactory(), new FileSystemWrapper(), new AbstractionFactory(), new JsonProvider())
        {
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="JSLintTask" /> class with custom services.
        /// </summary>
        /// <param name="jsLintFactory">The JSLint factory.</param>
        /// <param name="fileSystemWrapper">The file system wrapper.</param>
        /// <param name="abstractionFactory">The task logging helper factory.</param>
        /// <param name="jsonProvider">The JSON provider.</param>
        internal JSLintTask(IJSLintFactory jsLintFactory, IFileSystemWrapper fileSystemWrapper, IAbstractionFactory abstractionFactory, IJsonProvider jsonProvider)
        {
            this.jsLintFactory = jsLintFactory;
            this.fileSystemWrapper = fileSystemWrapper;
            this.jsonProvider = jsonProvider;

            this.LoggingHelper = abstractionFactory.CreateTaskLoggingHelper(this);
        }

        private delegate void Logger(string subcategory, string errorCode, string helpKeyword, string file, int lineNumber, int columnNumber, int endLineNumber, int endColumnNumber, string message, params object[] messageArgs);

        /// <summary>
        /// Gets or sets the source directory to be processed by JSLint.
        /// </summary>
        /// <value>
        /// The source files.
        /// </value>
        [Required]
        public string SourceDirectory { get; set; }

        /// <summary>
        /// Gets or sets a custom path to a JSLint.NET settings file if it is not located in the root of the source directory.
        /// </summary>
        /// <value>
        /// The settings file.
        /// </value>
        public string SettingsFile { get; set; }

        /// <summary>
        /// Gets or sets the path to the HTML report file.
        /// </summary>
        /// <value>
        /// The report file.
        /// </value>
        public string ReportFile { get; set; }

        /// <summary>
        /// Gets the error count.
        /// </summary>
        /// <value>
        /// The error count.
        /// </value>
        [Output]
        public int ErrorCount { get; private set; }

        /// <summary>
        /// Gets the count of files with errors.
        /// </summary>
        /// <value>
        /// The count of files with errors.
        /// </value>
        [Output]
        public int ErrorFileCount { get; private set; }

        /// <summary>
        /// Gets the processed file count.
        /// </summary>
        /// <value>
        /// The processed file count.
        /// </value>
        [Output]
        public int ProcessedFileCount { get; private set; }

        /// <summary>
        /// Gets the logging helper.
        /// </summary>
        /// <value>
        /// The logging helper.
        /// </value>
        internal ITaskLoggingHelper LoggingHelper { get; private set; }

        /// <summary>
        /// Executes the task.
        /// </summary>
        /// <returns>
        ///   <c>true</c> if the task successfully executed; otherwise, <c>false</c>.
        /// </returns>
        public override bool Execute()
        {
            var settings = this.GetSettings();
            var sourceFiles = this.GetSourceFiles(settings);

            if (sourceFiles.Length > 0)
            {
                var options = settings.Options;
                var logger = this.GetLogger(settings);
                var errorLimit = settings.ErrorLimitOrDefault();
                var fileLimit = settings.FileLimitOrDefault();
                var exceptions = 0;

                using (var context = this.jsLintFactory.CreateContext())
                using (var reportBuilder = this.jsLintFactory.CreateReportBuilder())
                {
                    reportBuilder.SourceDirectory = this.SourceDirectory;
                    reportBuilder.SettingsFile = settings.File;

                    foreach (var file in sourceFiles)
                    {
                        var relative = this.GetRelativePath(file);
                        var result = ExecutionHelper.Try(() =>
                        {
                            var source = this.fileSystemWrapper.ReadAllText(file, Encoding.UTF8);

                            return context.Lint(source, options);
                        });

                        if (result.Success)
                        {
                            var data = result.Data;
                            reportBuilder.AddFile(relative, data);

                            foreach (var error in data.Errors)
                            {
                                logger(
                                    subcategory: JSLintSubcategory,
                                    errorCode: null,
                                    helpKeyword: null,
                                    file: file,
                                    lineNumber: error.Line,
                                    columnNumber: error.Character,
                                    endLineNumber: 0,
                                    endColumnNumber: 0,
                                    message: error.Reason);
                            }

                            if (reportBuilder.ErrorCount >= errorLimit)
                            {
                                this.LoggingHelper.LogError(Resources.ErrorLimitReachedFormat, reportBuilder.ErrorCount);

                                break;
                            }
                        }
                        else
                        {
                            this.LoggingHelper.LogError(
                                Resources.ErrorEncounteredFormat,
                                relative,
                                Environment.NewLine,
                                result.Exception.Message);

                            exceptions += 1;

                            if (exceptions >= JSLintNetSettings.ExceptionLimit)
                            {
                                this.LoggingHelper.LogError(Resources.ExceptionLimitReachedFormat, exceptions);

                                break;
                            }
                        }

                        if (reportBuilder.ProcessedFileCount >= fileLimit)
                        {
                            this.LoggingHelper.LogError(Resources.FileLimitReachedFormat, reportBuilder.ProcessedFileCount);

                            break;
                        }
                    }

                    if (!string.IsNullOrEmpty(this.ReportFile))
                    {
                        this.fileSystemWrapper.WriteAllText(this.ReportFile, reportBuilder.ToString(), Encoding.UTF8);
                    }

                    this.ErrorCount = reportBuilder.ErrorCount;
                    this.ErrorFileCount = reportBuilder.ErrorFileCount;
                    this.ProcessedFileCount = reportBuilder.ProcessedFileCount;
                }
            }
            else
            {
                this.ErrorCount = 0;
                this.ErrorFileCount = 0;
                this.ProcessedFileCount = 0;
            }

            return settings.Output != Output.Error || this.ErrorCount == 0;
        }

        private string GetRelativePath(string file)
        {
            var relative = file.Substring(this.SourceDirectory.Length);
            var directorySeparator = Path.DirectorySeparatorChar.ToString();

            if (!relative.StartsWith(directorySeparator))
            {
                relative = directorySeparator + relative;
            }

            return relative;
        }

        private Logger GetLogger(JSLintNetSettings settings)
        {
            switch (settings.Output)
            {
                case Output.Warning:
                case Output.Message:
                    return this.LoggingHelper.LogWarning;
                default:
                    return this.LoggingHelper.LogError;
            }
        }

        private string[] GetSourceFiles(JSLintNetSettings settings)
        {
            var sourceFiles = this.fileSystemWrapper.GetFiles(this.SourceDirectory, "*", SearchOption.AllDirectories);
            var ignored = settings.NormalizeIgnore();

            return sourceFiles
                .Where(x => JSLint.CanLint(x) && !this.IsIgnored(x, ignored))
                .ToArray();
        }

        private bool IsIgnored(string file, IList<string> ignore)
        {
            var relative = this.GetRelativePath(file);

            return ignore.Any(x => relative.StartsWith(x, StringComparison.OrdinalIgnoreCase));
        }

        private JSLintNetSettings GetSettings()
        {
            JSLintNetSettings settings = null;
            var settingsPath = this.SettingsFile;

            if (string.IsNullOrEmpty(settingsPath))
            {
                settingsPath = Path.Combine(this.SourceDirectory, JSLintNetSettings.FileName);
            }

            if (this.fileSystemWrapper.FileExists(settingsPath))
            {
                var settingsSource = this.fileSystemWrapper.ReadAllText(settingsPath, Encoding.UTF8);

                settings = this.jsonProvider.DeserializeSettings(settingsSource);

                if (settings != null)
                {
                    settings.File = settingsPath;
                }
            }

            return settings ?? new JSLintNetSettings();
        }
    }
}
