﻿namespace JSLintNet.Console.Specifications
{
    using System;
    using JSLintNet.Abstractions;
    using JSLintNet.Json;
    using JSLintNet.QualityTools;
    using Moq;
    using Xunit;

    public class ConsoleOptionsFactoryUnit
    {
        public class Create : UnitBase
        {
            [Fact]
            public void Should_assume_help_given_null()
            {
                using (var testable = new CreateTestable())
                {
                    var actual = testable.Instance.Create(null);

                    Assert.True(actual.Help);
                }
            }

            [Fact]
            public void Should_assume_help_given_no_arguments()
            {
                using (var testable = new CreateTestable())
                {
                    var actual = testable.Instance.Create(new string[] { });

                    Assert.True(actual.Help);
                }
            }

            [Fact]
            public void Should_set_help_from_first_argument()
            {
                using (var testable = new CreateTestable())
                {
                    var actual = testable.Instance.Create(new string[] { "/?" });

                    Assert.True(actual.Help);
                }
            }

            [Fact]
            public void Should_throw_given_empty_source_directory()
            {
                using (var testable = new CreateTestable())
                {
                    Assert.Throws<ArgumentException>(() =>
                    {
                        var actual = testable.Instance.Create(new string[] { string.Empty });
                    });
                }
            }

            [Fact]
            public void Should_throw_given_invalid_first_argument()
            {
                using (var testable = new CreateTestable())
                {
                    Assert.Throws<ArgumentException>(() =>
                    {
                        var actual = testable.Instance.Create(new string[] { "-s" });
                    });
                }
            }

            [Fact]
            public void Should_set_source_directory_from_first_argument()
            {
                using (var testable = new CreateTestable())
                {
                    var actual = testable.Instance.Create(new string[] { "source directory" });

                    Assert.Equal("source directory", actual.SourceDirectory);
                    Assert.False(actual.Help);
                }
            }

            [Fact]
            public void Should_set_settings_file_from_arguments()
            {
                using (var testable = new CreateTestable())
                {
                    var actual = testable.Instance.Create(new string[]
                    {
                        "source directory",
                        "-settingsfile",
                        "settings file"
                    });

                    Assert.Equal("settings file", actual.SettingsFile);
                }
            }

            [Fact]
            public void Should_set_report_file_from_arguments()
            {
                using (var testable = new CreateTestable())
                {
                    var actual = testable.Instance.Create(new string[]
                    {
                        "source directory",
                        "-reportfile",
                        "report file"
                    });

                    Assert.Equal("report file", actual.ReportFile);
                }
            }

            [Fact]
            public void Should_set_log_level_from_arguments()
            {
                using (var testable = new CreateTestable())
                {
                    var actual = testable.Instance.Create(new string[]
                    {
                        "source directory",
                        "-loglevel",
                        "silent"
                    });

                    Assert.Equal(LogLevel.Silent, actual.LogLevel);
                }
            }

            [Fact]
            public void Should_set_all_from_abbreviations()
            {
                using (var testable = new CreateTestable())
                {
                    var actual = testable.Instance.Create(new string[]
                    {
                        "source directory",
                        "/l",
                        "1",
                        "/r",
                        "report file",
                        "/s",
                        "settings file"
                    });

                    Assert.Equal("settings file", actual.SettingsFile);
                    Assert.Equal("report file", actual.ReportFile);
                    Assert.Equal(LogLevel.Verbose, actual.LogLevel);
                }
            }

            [Fact]
            public void Should_throw_given_key_without_value()
            {
                using (var testable = new CreateTestable())
                {
                    Assert.Throws<ArgumentException>(() =>
                    {
                        var actual = testable.Instance.Create(new string[]
                        {
                            "source directory",
                            "/r",
                            "/s",
                            "settings file"
                        });
                    });
                }
            }

            private class CreateTestable : ConsoleOptionsFactoryUnitTestableBase
            {
                public CreateTestable()
                {
                    this.BeforeInit += this.OnBeforeInit;
                }

                private void OnBeforeInit(object sender, EventArgs e)
                {
                    this.FileSystemWrapperMock
                        .Setup(x => x.ResolveDirectory(It.IsAny<string>()))
                        .Returns((string x) => x);
                }
            }
        }

        private abstract class ConsoleOptionsFactoryUnitTestableBase : TestableBase<ConsoleOptionsFactory>
        {
            public ConsoleOptionsFactoryUnitTestableBase()
            {
                this.FileSystemWrapperMock = this.AutoMocker.Mock<IFileSystemWrapper>();
                this.JsonProviderMock = this.AutoMocker.Mock<IJsonProvider>();
            }

            public Mock<IFileSystemWrapper> FileSystemWrapperMock { get; set; }

            public Mock<IJsonProvider> JsonProviderMock { get; set; }
        }
    }
}
