﻿namespace JSLintNet.Specifications
{
    using System;
    using System.IO;
    using JSLintNet.QualityTools;
    using Noesis.Javascript;
    using Xunit;

    public class JSLintContextIntegration : IntegrationBase
    {
        public class Constructor : IntegrationBase
        {
            [Fact]
            public void Should_throw_given_old_jslint_edition_on_file_system()
            {
                try
                {
                    ConstructorHelper.CreateJSLint("var JSLINT = { edition: '2000-01-01' };");

                    var exception = Assert.Throws<Exception>(() =>
                    {
                        using (new JSLintContext())
                        {
                        }
                    });

                    Assert.Contains("edition", exception.Message);
                }
                finally
                {
                    ConstructorHelper.DeleteJSLint();
                }
            }

            [Fact]
            public void Should_throw_given_invalid_jslint_from_file_system()
            {
                try
                {
                    ConstructorHelper.CreateJSLint("not.actually.jslint;");

                    Assert.Throws<JavascriptException>(() =>
                    {
                        using (new JSLintContext())
                        {
                        }
                    });
                }
                finally
                {
                    ConstructorHelper.DeleteJSLint();
                }
            }

            [Fact]
            public void Should_not_throw_given_new_jslint_edition_from_file_system()
            {
                try
                {
                    ConstructorHelper.CreateJSLint("var JSLINT = { edition: '3000-01-01' };");

                    using (new JSLintContext())
                    {
                    }
                }
                finally
                {
                    ConstructorHelper.DeleteJSLint();
                }
            }

            [Fact]
            public void Should_not_throw_given_equal_jslint_edition_from_file_system()
            {
                try
                {
                    ConstructorHelper.CreateJSLint(string.Format("var JSLINT = {{ edition: '{0}' }};", AssemblyInfo.Edition));

                    using (new JSLintContext())
                    {
                    }
                }
                finally
                {
                    ConstructorHelper.DeleteJSLint();
                }
            }

            private static class ConstructorHelper
            {
                public static readonly string JSLintPath = GetJSLintPath();

                public static string GetJSLintPath()
                {
                    var path = typeof(JSLintContext).Assembly.Location;
                    path = Path.GetDirectoryName(path);
                    path = Path.Combine(path, "jslint.js");

                    return path;
                }

                public static void CreateJSLint(string source)
                {
                    File.WriteAllText(JSLintPath, source);
                }

                public static void DeleteJSLint()
                {
                    File.Delete(JSLintPath);
                }
            }
        }

        public class Lint : IntegrationBase
        {
            [Fact]
            public void Should_fail_given_invalid_source()
            {
                using (var instance = new JSLintContext())
                {
                    var result = instance.Lint("some invalid source");

                    Assert.NotEmpty(result.Errors);
                }
            }

            [Fact]
            public void Should_have_stopping_error_when_maximum_reached()
            {
                using (var instance = new JSLintContext())
                {
                    var result = instance.Lint(
                        "var GLOBAL_VARIABLE",
                        new JSLintOptions()
                        {
                            MaximumErrors = 1
                        });

                    Assert.Equal(2, result.Errors.Count);
                    Assert.True(result.HasStoppingError);
                }
            }

            [Fact]
            public void Should_not_throw_given_a_use_param_error()
            {
                using (var instance = new JSLintContext())
                {
                    var result = instance.Lint(
                        @"function sample(a, b, c, d) { 'use strict'; return arguments[3]; }",
                        new JSLintOptions()
                        {
                            TolerateUnusedParameters = true
                        });

                    Assert.Equal(1, result.Errors.Count);
                }
            }

            [Fact]
            public void Should_succeed_given_valid_source()
            {
                using (var instance = new JSLintContext())
                {
                    var result = instance.Lint(@"
var ValidSource = (function () {
    'use strict';

    return {
        memberProperty: 'value',
        memberFunction: function (arg) {
            return this.memberProperty + arg;
        }
    };
}());");

                    Assert.Empty(result.Errors);
                }
            }
        }
    }
}
