﻿namespace JSLintNet.Specifications
{
    using System.Collections.Generic;
    using JSLintNet.Abstractions;
    using JSLintNet.Json;
    using JSLintNet.Models;
    using JSLintNet.QualityTools;
    using Moq;
    using Xunit;

    public class JSLintContextUnit
    {
        public class Constructor : UnitBase
        {
            [Fact]
            public void Should_create_a_new_javascript_context()
            {
                using (var testable = new ConstructorTestable())
                {
                    testable.Init();

                    testable.AbstractionFactory.Verify(x => x.CreateJavaScriptContext());
                }
            }

            [Fact]
            public void Should_setup_jslint_inside_a_javascript_context()
            {
                using (var testable = new ConstructorTestable())
                {
                    testable.Init();

                    Assert.True(testable.ContextRuns[0].StartsWith("// jslint.js"));
                }
            }

            private class ConstructorTestable : JSLintContextTestableBase
            {
            }
        }

        public class Dispose : UnitBase
        {
            [Fact]
            public void Should_dispose_of_javascript_context()
            {
                using (var testable = new DisposeTestable())
                {
                    testable.Instance.Dispose();

                    testable.JavaScriptContextMock.Verify(x => x.Dispose());
                }
            }

            private class DisposeTestable : JSLintContextTestableBase
            {
            }
        }

        public class Lint : UnitBase
        {
            [Fact]
            public void Should_pass_source_to_javascript_context()
            {
                using (var testable = new LintTestable())
                {
                    testable.Instance.Lint("some source");

                    testable.JavaScriptContextMock.Verify(x => x.SetParameter(JSLintParameters.SourceParameter, "some source"));
                }
            }

            [Fact]
            public void Should_pass_serialized_options_to_javascript_context()
            {
                using (var testable = new LintTestable())
                {
                    testable.JsonProviderMock
                    .Setup(x => x.SerializeOptions(It.IsAny<JSLintOptions>()))
                    .Returns(@"{""some"":""json""}");

                    var options = new JSLintOptions()
                    {
                        TolerateStupidPractices = true
                    };

                    testable.Instance.Lint("some source", options);

                    testable.JavaScriptContextMock
                        .Verify(x => x.SetParameter(
                            JSLintParameters.OptionsParameter,
                            @"{""some"":""json""}"));
                }
            }

            [Fact]
            public void Should_run_JSLint_over_source()
            {
                using (var testable = new LintTestable())
                {
                    testable.Instance.Lint("some source");

                    Assert.True(testable.ContextRuns[1].StartsWith("JSLINT(" + JSLintParameters.SourceParameter));
                }
            }

            [Fact]
            public void Should_run_JSLint_with_options()
            {
                using (var testable = new LintTestable())
                {
                    testable.Instance.Lint("some source");

                    Assert.Contains(JSLintParameters.OptionsParameter, testable.ContextRuns[1]);
                }
            }

            [Fact]
            public void Should_fetch_data_from_JSLint_run()
            {
                using (var testable = new LintTestable())
                {
                    testable.Instance.Lint("some source");

                    Assert.Contains("JSLINT.data()", testable.ContextRuns[2]);
                }
            }

            private class LintTestable : JSLintContextTestableBase
            {
            }
        }

        private abstract class JSLintContextTestableBase : TestableBase<JSLintContext>
        {
            public JSLintContextTestableBase()
            {
                this.ContextRuns = new List<string>();
                this.AbstractionFactory = this.AutoMocker.Mock<IAbstractionFactory>();
                this.JsonProviderMock = this.AutoMocker.Mock<IJsonProvider>();

                this.BeforeInit += this.OnBeforeInit;
            }

            public Mock<IAbstractionFactory> AbstractionFactory { get; set; }

            public Mock<IJsonProvider> JsonProviderMock { get; set; }

            public Mock<IJavaScriptContext> JavaScriptContextMock { get; set; }

            public List<string> ContextRuns { get; set; }

            private void OnBeforeInit(object sender, System.EventArgs e)
            {
                this.JsonProviderMock
                    .Setup(x => x.DeserializeData(It.IsAny<string>()))
                    .Returns(() => new JSLintData());

                this.JavaScriptContextMock = this.AutoMocker.Mock<IJavaScriptContext>();

                this.AbstractionFactory
                    .Setup(x => x.CreateJavaScriptContext())
                    .Returns(this.JavaScriptContextMock.Object);

                this.JavaScriptContextMock
                    .Setup(x => x.Run(It.IsAny<string>()))
                    .Callback((string x) => this.ContextRuns.Add(x))
                    .Returns(new Dictionary<string, object>());
            }
        }
    }
}
