﻿namespace JSLintNet.Specifications
{
    using System;
    using System.IO;
    using System.Text.RegularExpressions;
    using JSLintNet.QualityTools;
    using JSLintNet.QualityTools.Helpers;
    using Xunit;

    public class JSLintTaskIntegration : IntegrationBase
    {
        private static readonly string MSBuildExecutable = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.Windows), @"Microsoft.NET\Framework\v4.0.30319\MSBuild.exe");

        private static readonly string ProjectRoot = Path.GetFullPath(Path.Combine(Environment.CurrentDirectory, @"..\..\Resources"));

        private static readonly Regex ErrorCountPattern = new Regex(@"JSLINTERRORCOUNT=(?<Count>[\d]+)=JSLINTERRORCOUNT", RegexOptions.Compiled);

        private static readonly Regex ErrorFileCountPattern = new Regex(@"JSLINTERRORFILECOUNT=(?<Count>[\d]+)=JSLINTERRORFILECOUNT", RegexOptions.Compiled);

        private static readonly Regex ProcessedFileCountPattern = new Regex(@"JSLINTPROCESSEDFILECOUNT=(?<Count>[\d]+)=JSLINTPROCESSEDFILECOUNT", RegexOptions.Compiled);

        [Fact]
        public void Should_fail_with_correct_counts_using_errors_project()
        {
            var actual = ExecuteMSBuildProject("Errors");

            Assert.False(actual.Success);
            Assert.Equal(12, actual.ErrorCount);
            Assert.Equal(3, actual.ErrorFileCount);
            Assert.Equal(5, actual.ProcessedFileCount);
        }

        [Fact]
        public void Should_succeed_with_correct_counts_using_warnings_project()
        {
            var actual = ExecuteMSBuildProject("Warnings");

            Assert.True(actual.Success);
            Assert.Equal(12, actual.ErrorCount);
            Assert.Equal(3, actual.ErrorFileCount);
            Assert.Equal(5, actual.ProcessedFileCount);
        }

        [Fact]
        public void Should_fail_when_source_directory_property_omitted()
        {
            var actual = ExecuteMSBuildProject("NoSourceDirectory");

            Assert.False(actual.Success);
            Assert.Contains("task was not given a value for the required parameter \"SourceDirectory\"", actual.Output);
        }

        [Fact]
        public void Should_save_html_report_with_report_file_property()
        {
            var actual = ExecuteMSBuildProject("HtmlReport");

            var reportPath = Path.Combine(ProjectRoot, "JSLintReport.html");
            Assert.True(File.Exists(reportPath));
            File.Delete(reportPath);
        }

        private static int ParseCount(Regex pattern, string input)
        {
            var match = pattern.Match(input);

            if (match.Success && match.Groups["Count"].Success)
            {
                int count;

                if (int.TryParse(match.Groups["Count"].Value, out count))
                {
                    return count;
                }
            }

            return -1;
        }

        private static JSLintTaskResult ExecuteMSBuildProject(string projectName)
        {
            var result = ProcessHelper.Execute(MSBuildExecutable, projectName + ".proj", ProjectRoot);
            var exitCode = result.ExitCode;
            var output = result.Output;

            return new JSLintTaskResult()
            {
                ExitCode = result.ExitCode,
                Output = result.Output,
                Error = result.Error,
                ErrorCount = ParseCount(ErrorCountPattern, output),
                ErrorFileCount = ParseCount(ErrorFileCountPattern, output),
                ProcessedFileCount = ParseCount(ProcessedFileCountPattern, output),
                Success = exitCode == 0 && output.Contains("Build succeeded.")
            };
        }

        private class JSLintTaskResult : ProcessResult
        {
            public int ErrorCount { get; set; }

            public int ErrorFileCount { get; set; }

            public int ProcessedFileCount { get; set; }

            public bool Success { get; set; }
        }
    }
}
