﻿namespace JSLintNet.Specifications
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Text;
    using JSLintNet;
    using JSLintNet.Abstractions;
    using JSLintNet.Json;
    using JSLintNet.Models;
    using JSLintNet.Properties;
    using JSLintNet.QualityTools;
    using JSLintNet.QualityTools.Fakes;
    using Microsoft.Build.Framework;
    using Moq;
    using Xunit;

    public class JSLintTaskUnit
    {
        public class Execute : UnitBase
        {
            [Fact]
            public void Should_return_true_when_no_files_found()
            {
                using (var testable = new ExecuteTestable())
                {
                    var actual = testable.Instance.Execute();

                    Assert.True(actual);
                }
            }

            [Fact]
            public void Should_return_true_when_no_files_contain_errors()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file.js", 0);

                    var actual = testable.Instance.Execute();

                    Assert.True(actual);
                }
            }

            [Fact]
            public void Should_return_false_when_one_error_found()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file.js", 1);

                    var actual = testable.Instance.Execute();

                    Assert.False(actual);
                }
            }

            [Fact]
            public void Should_return_false_when_many_errors_found()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file1.js", 5);
                    testable.SetupJSLintFile("file2.js", 0);
                    testable.SetupJSLintFile("file3.js", 3);

                    var actual = testable.Instance.Execute();

                    Assert.False(actual);
                }
            }

            [Fact]
            public void Should_return_true_when_errors_found_but_treating_errors_as_warnings()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file.js", 2);
                    testable.Settings.Output = Output.Warning;
                    testable.SettingsExist = true;

                    var actual = testable.Instance.Execute();

                    Assert.True(actual);
                }
            }

            [Fact]
            public void Should_lint_the_content_of_each_file_provided()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file1.js", 0);
                    testable.SetupJSLintFile("jsfile2.js", 0);

                    testable.Instance.Execute();

                    testable.JSLintContextMock.Verify(x => x.Lint("file1.js contents", It.IsAny<JSLintOptions>()));
                    testable.JSLintContextMock.Verify(x => x.Lint("jsfile2.js contents", It.IsAny<JSLintOptions>()));
                    testable.JSLintContextMock.Verify(x => x.Lint(It.IsAny<string>(), It.IsAny<JSLintOptions>()), Times.Exactly(2));
                }
            }

            [Fact]
            public void Should_log_each_jslint_error_as_a_task_error()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file1.js", 2);
                    testable.SetupJSLintFile("jsfile2.js", 1);

                    testable.Instance.Execute();

                    testable.LoggingHelperMock.Verify(x => x.LogError(It.IsAny<string>(), null, null, It.Is<string>(y => y.EndsWith("file1.js")), 1, 1, 0, 0, "file1.js message 1"));
                    testable.LoggingHelperMock.Verify(x => x.LogError(It.IsAny<string>(), null, null, It.Is<string>(y => y.EndsWith("file1.js")), 2, 2, 0, 0, "file1.js message 2"));
                    testable.LoggingHelperMock.Verify(x => x.LogError(It.IsAny<string>(), null, null, It.Is<string>(y => y.EndsWith("jsfile2.js")), 1, 1, 0, 0, "jsfile2.js message 1"));
                    testable.LoggingHelperMock.Verify(x => x.LogWarning(It.IsAny<string>(), null, null, It.IsAny<string>(), It.IsAny<int>(), It.IsAny<int>(), 0, 0, It.IsAny<string>()), Times.Never());
                }
            }

            [Fact]
            public void Should_log_each_jslint_error_as_a_task_warning_when_treating_errors_as_warnings()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file1.js", 2);
                    testable.SetupJSLintFile("jsfile2.js", 1);
                    testable.Settings.Output = Output.Warning;
                    testable.SettingsExist = true;

                    testable.Instance.Execute();

                    testable.LoggingHelperMock.Verify(x => x.LogWarning(It.IsAny<string>(), null, null, It.Is<string>(y => y.EndsWith("file1.js")), 1, 1, 0, 0, "file1.js message 1"));
                    testable.LoggingHelperMock.Verify(x => x.LogWarning(It.IsAny<string>(), null, null, It.Is<string>(y => y.EndsWith("file1.js")), 2, 2, 0, 0, "file1.js message 2"));
                    testable.LoggingHelperMock.Verify(x => x.LogWarning(It.IsAny<string>(), null, null, It.Is<string>(y => y.EndsWith("jsfile2.js")), 1, 1, 0, 0, "jsfile2.js message 1"));
                    testable.LoggingHelperMock.Verify(x => x.LogError(It.IsAny<string>(), null, null, It.IsAny<string>(), It.IsAny<int>(), It.IsAny<int>(), 0, 0, It.IsAny<string>()), Times.Never());
                }
            }

            [Fact]
            public void Should_set_error_count_to_total_number_of_error()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file1.js", 2);
                    testable.SetupJSLintFile("file2.js", 1);
                    testable.SetupJSLintFile("file3.js", 5);
                    testable.SetupJSLintFile("file4.js", 2);

                    testable.Instance.Execute();

                    Assert.Equal(testable.Instance.ErrorCount, 10);
                }
            }

            [Fact]
            public void Should_set_error_file_count_to_total_number_of_files_with_errors()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file1.js", 2);
                    testable.SetupJSLintFile("file2.js", 0);
                    testable.SetupJSLintFile("file3.js", 3);

                    testable.Instance.Execute();

                    Assert.Equal(2, testable.Instance.ErrorFileCount);
                }
            }

            [Fact]
            public void Should_set_process_file_count_to_total_count_of_files_processed()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file1.js", 3);
                    testable.SetupJSLintFile("file2.js", 0);
                    testable.SetupJSLintFile("file3.js", 0);
                    testable.SetupJSLintFile("file4.js", 1);

                    testable.Instance.Execute();

                    Assert.Equal(4, testable.Instance.ProcessedFileCount);
                }
            }

            [Fact]
            public void Should_not_try_to_create_a_reporter_if_no_source_files_exist()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.Instance.Execute();

                    testable.JSLintFactoryMock.Verify(x => x.CreateReportBuilder(), Times.Never());
                }
            }

            [Fact]
            public void Should_try_to_create_a_report_builder()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file.js", 0);

                    testable.Instance.Execute();

                    testable.JSLintFactoryMock.Verify(x => x.CreateReportBuilder());
                }
            }

            [Fact]
            public void Should_add_all_files_with_data_to_report_builder_if_it_exists()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file1.js", 3);
                    testable.SetupJSLintFile("file2.js", 5);

                    testable.Instance.Execute();

                    testable.JSLintReportBuilderMock.Verify(x => x.AddFile(It.Is<string>(y => y.EndsWith("file1.js")), It.IsAny<IJSLintData>()));
                    testable.JSLintReportBuilderMock.Verify(x => x.AddFile(It.Is<string>(y => y.EndsWith("file2.js")), It.IsAny<IJSLintData>()));
                }
            }

            [Fact]
            public void Should_save_report_as_UTF8_if_it_exists()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file.js", 0);

                    testable.Instance.ReportFile = "REPORTFILE";
                    testable.Instance.Execute();

                    testable.JSLintReportBuilderMock.Verify(x => x.ToString());
                    testable.FileSystemWrapperMock.Verify(x => x.WriteAllText("REPORTFILE", "REPORTRESULT", Encoding.UTF8));
                }
            }

            [Fact]
            public void Should_stop_when_default_error_limit_reached()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file.js", 0);
                    testable.ErrorCount = JSLintNetSettings.DefaultErrorLimit;

                    testable.Instance.Execute();

                    testable.LoggingHelperMock.Verify(x => x.LogError(Resources.ErrorLimitReachedFormat, JSLintNetSettings.DefaultErrorLimit));
                }
            }

            [Fact]
            public void Should_stop_when_hard_exception_limit_reached()
            {
                using (var testable = new ExecuteTestable())
                {
                    for (int i = 0; i < JSLintNetSettings.ExceptionLimit; i++)
                    {
                        var filePath = Path.Combine(testable.SourceDirectory, "file" + i + ".js");
                        testable.SourceFiles.Add(filePath);
                    }

                    testable.JSLintContextMock
                        .Setup(x => x.Lint(It.IsAny<string>(), It.IsAny<JSLintOptions>()))
                        .Throws<Exception>();

                    testable.Instance.Execute();

                    testable.LoggingHelperMock.Verify(x => x.LogError(Resources.ExceptionLimitReachedFormat, JSLintNetSettings.ExceptionLimit));
                }
            }

            [Fact]
            public void Should_stop_when_default_file_limit_reached()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.SetupJSLintFile("file.js", 0);
                    testable.ProcessedFileCount = JSLintNetSettings.DefaultFileLimit;

                    testable.Instance.Execute();

                    testable.LoggingHelperMock.Verify(x => x.LogError(Resources.FileLimitReachedFormat, JSLintNetSettings.DefaultFileLimit));
                }
            }

            [Fact]
            public void Should_stop_when_custom_error_limit_reached()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.Settings.ErrorLimit = 10;
                    testable.SettingsExist = true;
                    testable.SetupJSLintFile("file.js", 11);

                    testable.Instance.Execute();

                    testable.LoggingHelperMock.Verify(x => x.LogError(Resources.ErrorLimitReachedFormat, 11));
                }
            }

            [Fact]
            public void Should_stop_when_custom_file_limit_reached()
            {
                using (var testable = new ExecuteTestable())
                {
                    testable.Settings.FileLimit = 10;
                    testable.SettingsExist = true;

                    testable.SetupJSLintFile("file.js", 0);
                    testable.ProcessedFileCount = 11;

                    testable.Instance.Execute();

                    testable.LoggingHelperMock.Verify(x => x.LogError(Resources.FileLimitReachedFormat, 11));
                }
            }

            private class ExecuteTestable : JSLintTaskTestableBase
            {
                public ExecuteTestable()
                {
                    this.SourceDirectory = @"SOURCE:DIRECTORY\";
                    this.JSLintContextMock = new Mock<IJSLintContext>();
                    this.JSLintReportBuilderMock = new Mock<IJSLintReportBuilder>();
                    this.SourceFiles = new List<string>();
                    this.Settings = new JSLintNetSettings();

                    this.BeforeInit += this.OnBeforeInit;

                    this.AfterInit += this.OnAfterInit;
                }

                public Mock<IJSLintContext> JSLintContextMock { get; set; }

                public Mock<IJSLintReportBuilder> JSLintReportBuilderMock { get; set; }

                public int ProcessedFileCount { get; set; }

                public int ErrorFileCount { get; set; }

                public int ErrorCount { get; set; }

                public string SourceDirectory { get; set; }

                public List<string> SourceFiles { get; set; }

                public bool SettingsExist { get; set; }

                public JSLintNetSettings Settings { get; set; }

                public void SetupJSLintFile(string fileName, int errorCount = 0)
                {
                    using (var testable = new ExecuteTestable())
                    {
                        var filePath = Path.Combine(this.SourceDirectory, fileName);
                        this.SourceFiles.Add(filePath);

                        var contents = fileName + " contents";
                        var data = new JSLintDataFake(fileName, errorCount);

                        this.FileSystemWrapperMock
                            .Setup(x => x.ReadAllText(filePath, It.IsAny<Encoding>()))
                            .Returns(contents);

                        this.JSLintContextMock
                            .Setup(x => x.Lint(contents, It.IsAny<JSLintOptions>()))
                            .Returns(data);

                        this.ProcessedFileCount += 1;

                        if (errorCount > 0)
                        {
                            this.ErrorFileCount += 1;
                            this.ErrorCount += errorCount;
                        }
                    }
                }

                private void OnBeforeInit(object sender, EventArgs e)
                {
                    this.JSLintFactoryMock
                        .Setup(x => x.CreateContext())
                        .Returns(this.JSLintContextMock.Object);

                    this.JSLintFactoryMock
                        .Setup(x => x.CreateReportBuilder())
                        .Returns(this.JSLintReportBuilderMock.Object);

                    this.JSLintReportBuilderMock
                        .Setup(x => x.ToString())
                        .Returns("REPORTRESULT");

                    this.JSLintReportBuilderMock
                        .SetupGet(x => x.ErrorCount)
                        .Returns(() => this.ErrorCount);

                    this.JSLintReportBuilderMock
                        .SetupGet(x => x.ErrorFileCount)
                        .Returns(() => this.ErrorFileCount);

                    this.JSLintReportBuilderMock
                        .SetupGet(x => x.ProcessedFileCount)
                        .Returns(() => this.ProcessedFileCount);

                    this.FileSystemWrapperMock
                        .Setup(x => x.GetFiles(this.SourceDirectory, It.IsAny<string>(), SearchOption.AllDirectories))
                        .Returns(() => this.SourceFiles.ToArray());

                    var settingsPath = Path.Combine(this.SourceDirectory, JSLintNetSettings.FileName);
                    this.FileSystemWrapperMock
                        .Setup(x => x.FileExists(settingsPath))
                        .Returns(() => this.SettingsExist);

                    this.JsonProviderMock
                        .Setup(x => x.DeserializeSettings(It.IsAny<string>()))
                        .Returns(this.Settings);
                }

                private void OnAfterInit(object sender, EventArgs e)
                {
                    this.Instance.SourceDirectory = this.SourceDirectory;
                }
            }
        }

        private abstract class JSLintTaskTestableBase : TestableBase<JSLintTask>
        {
            public JSLintTaskTestableBase()
            {
                this.FileSystemWrapperMock = this.AutoMocker.Mock<IFileSystemWrapper>();
                this.AbstractionFactoryMock = this.AutoMocker.Mock<IAbstractionFactory>();
                this.JSLintFactoryMock = this.AutoMocker.Mock<IJSLintFactory>();
                this.JsonProviderMock = this.AutoMocker.Mock<IJsonProvider>();
                this.LoggingHelperMock = new Mock<ITaskLoggingHelper>();

                this.BeforeInit += this.OnBeforeInit;
            }

            public Mock<IFileSystemWrapper> FileSystemWrapperMock { get; set; }

            public Mock<IAbstractionFactory> AbstractionFactoryMock { get; set; }

            public Mock<IJsonProvider> JsonProviderMock { get; set; }

            public Mock<ITaskLoggingHelper> LoggingHelperMock { get; set; }

            public Mock<IJSLintFactory> JSLintFactoryMock { get; set; }

            private void OnBeforeInit(object sender, EventArgs e)
            {
                this.AbstractionFactoryMock
                    .Setup(x => x.CreateTaskLoggingHelper(It.IsAny<ITask>()))
                    .Returns(this.LoggingHelperMock.Object);
            }
        }
    }
}
