﻿namespace JSLintNet.Specifications.Json
{
    using JSLintNet.Json;
    using JSLintNet.QualityTools;
    using JSLintNet.QualityTools.Helpers;
    using Xunit;

    public class JsonProviderIntegration
    {
        public abstract class JsonProviderIntegrationBase : IntegrationBase
        {
            public JsonProviderIntegrationBase()
            {
                this.Instance = new JsonProvider();
            }

            internal JsonProvider Instance { get; private set; }
        }

        public class SerializeSettings : JsonProviderIntegrationBase
        {
            [Fact]
            public void Should_correctly_serialize_output_enum()
            {
                var settings = new JSLintNetSettings()
                {
                    Output = Output.Warning
                };

                var actual = this.Instance.SerializeSettings(settings);

                Assert.Contains(@"""output"": ""Warning""", actual);
            }

            [Fact]
            public void Should_correctly_serialize_booleans()
            {
                var settings = new JSLintNetSettings()
                {
                    RunOnBuild = true
                };

                var actual = this.Instance.SerializeSettings(settings);

                Assert.Contains(@"""runOnSave"": false", actual);
                Assert.Contains(@"""runOnBuild"": true", actual);
                Assert.Contains(@"""cancelBuild"": false", actual);
            }

            [Fact]
            public void Should_correctly_serialize_ignore_list()
            {
                var settings = new JSLintNetSettings();

                settings.Ignore.Add(@"\mypath\");
                settings.Ignore.Add(@"\ignore\libraries\*.js");

                var actual = this.Instance.SerializeSettings(settings);

                Expect.Matches(@"""ignore"": \[\s*""\\\\mypath\\\\"",\s*""\\\\ignore\\\\libraries\\\\\*\.js""\s*\]", actual);
            }

            [Fact]
            public void Should_correctly_serialize_options()
            {
                var settings = new JSLintNetSettings()
                {
                    Options = new JSLintOptions()
                    {
                        AssumeBrowser = true,
                        TolerateUnusedParameters = true
                    }
                };
                settings.Options.PredefinedGlobals.Add("MyGlobal", false);

                var actual = this.Instance.SerializeSettings(settings);

                Expect.Matches(@"""options"": \{\s*""predef"": \{\s*""MyGlobal"": false\s*\},\s*""browser"": true,\s*""unparam"": true\s*\}", actual);
            }
        }

        public class DeserializeSettings : JsonProviderIntegrationBase
        {
            [Fact]
            public void Should_correctly_deserialize_output_enum()
            {
                var value = @"{""output"":""message""}";

                var actual = this.Instance.DeserializeSettings(value);

                Assert.Equal(Output.Message, actual.Output);
            }

            [Fact]
            public void Should_correctly_deserialize_booleans()
            {
                var value = @"{""runOnBuild"":true}";

                var actual = this.Instance.DeserializeSettings(value);

                Assert.False(actual.RunOnSave);
                Assert.True(actual.RunOnBuild);
                Assert.False(actual.CancelBuild);
            }

            [Fact]
            public void Should_correctly_deserialize_ignore_list()
            {
                var value = @"{""ignore"":[""\\mypath\\"",""\\ignore\\libraries\\*.js""]}";

                var actual = this.Instance.DeserializeSettings(value);

                Assert.Contains(@"\mypath\", actual.Ignore);
                Assert.Contains(@"\ignore\libraries\*.js", actual.Ignore);
                Assert.Equal(2, actual.Ignore.Count);
            }

            [Fact]
            public void Should_correctly_deserialize_options()
            {
                var value = @"{""options"":{""predef"":{""MyGlobal"":true},""browser"":true,""unparam"":true}}";

                var actual = this.Instance.DeserializeSettings(value);
                var options = actual.Options;

                Assert.True(options.PredefinedGlobals["MyGlobal"]);
                Assert.True(options.AssumeBrowser.Value);
                Assert.True(options.TolerateUnusedParameters.Value);
            }
        }
    }
}
