﻿namespace JSLintNet.VisualStudio.Specifications.EventControllers
{
    using System;
    using System.Collections.Generic;
    using EnvDTE;
    using JSLintNet.QualityTools;
    using JSLintNet.VisualStudio.EventControllers;
    using JSLintNet.VisualStudio.Properties;
    using JSLintNet.VisualStudio.Specifications.Fakes;
    using Moq;
    using Xunit;

    public class BuildEventControllerUnit
    {
        public class OnBuildProjectConfigBegin : UnitBase
        {
            [Fact]
            public void Should_not_run_for_clean_build()
            {
                using (var testable = new OnBuildProjectConfigBeginTestable())
                {
                    testable.Instance.Initialize();
                    testable.BuildEventsMock.Raise(x => x.OnBuildBegin += null, default(vsBuildScope), vsBuildAction.vsBuildActionClean);
                    testable.BuildEventsMock.Raise(x => x.OnBuildProjConfigBegin += null, "Some Name", string.Empty, string.Empty, string.Empty);

                    testable.EnvironmentMock.VerifyGet(x => x.Solution, Times.Never());
                }
            }

            [Fact]
            public void Should_not_run_when_project_is_not_found()
            {
                using (var testable = new OnBuildProjectConfigBeginTestable())
                {
                    testable.Instance.Initialize();
                    testable.BuildEventsMock.Raise(x => x.OnBuildProjConfigBegin += null, "Not Found", string.Empty, string.Empty, string.Empty);

                    testable.EnvironmentMock.VerifyGet(x => x.Solution);
                    testable.VisualStudioJSLintProviderMock.Verify(x => x.LoadSettings(It.IsAny<Project>()), Times.Never());
                }
            }

            [Fact]
            public void Should_not_run_when_run_on_build_is_false()
            {
                using (var testable = new OnBuildProjectConfigBeginTestable())
                {
                    testable.Settings.RunOnBuild = false;

                    testable.Instance.Initialize();
                    testable.BuildEventsMock.Raise(x => x.OnBuildProjConfigBegin += null, testable.ProjectMock.Object.UniqueName, string.Empty, string.Empty, string.Empty);

                    testable.VisualStudioJSLintProviderMock.Verify(x => x.LoadSettings(testable.ProjectMock.Object));
                    testable.ProjectMock.VerifyGet(x => x.ProjectItems, Times.Never());
                }
            }

            [Fact]
            public void Should_find_and_process_all_lintables_when_run_on_build_is_true()
            {
                using (var testable = new OnBuildProjectConfigBeginTestable())
                {
                    testable.Settings.RunOnBuild = true;

                    testable.Instance.Initialize();
                    testable.BuildEventsMock.Raise(x => x.OnBuildProjConfigBegin += null, testable.ProjectMock.Object.UniqueName, string.Empty, string.Empty, string.Empty);

                    Assert.NotNull(testable.LastItemsLinted);
                    Assert.Equal(2, testable.LastItemsLinted.Count);
                    testable.EnvironmentMock.Verify(x => x.ExecuteCommand("Build.Cancel", string.Empty), Times.Never());
                }
            }

            [Fact]
            public void Should_cancel_build_when_errors_found_and_cancel_build_is_true()
            {
                using (var testable = new OnBuildProjectConfigBeginTestable())
                {
                    testable.Settings.RunOnBuild = true;
                    testable.Settings.CancelBuild = true;
                    testable.ErrorsFound = 222;

                    testable.Instance.Initialize();
                    testable.BuildEventsMock.Raise(x => x.OnBuildProjConfigBegin += null, testable.ProjectMock.Object.UniqueName, string.Empty, string.Empty, string.Empty);

                    testable.EnvironmentMock.Verify(x => x.ExecuteCommand("Build.Cancel", string.Empty));
                    testable.ErrorListProviderMock.Verify(x => x.AddCustomError(Resources.BuildCancelled));
                }
            }

            private class OnBuildProjectConfigBeginTestable : BuildEventControllerTestableBase
            {
                public OnBuildProjectConfigBeginTestable()
                {
                    this.SolutionMock = new Mock<Solution>();
                    this.ProjectsFake = new ProjectsFake();
                    this.ProjectMock = new Mock<Project>();
                    this.ProjectItemsFake = new ProjectItemsFake();
                    this.Settings = new JSLintNetSettings();

                    this.BeforeInit += this.OnBeforeInit;
                }

                public Mock<Solution> SolutionMock { get; set; }

                public ProjectsFake ProjectsFake { get; set; }

                public Mock<Project> ProjectMock { get; set; }

                public ProjectItemsFake ProjectItemsFake { get; set; }

                public IList<ProjectItem> LastItemsLinted { get; set; }

                public int ErrorsFound { get; set; }

                internal JSLintNetSettings Settings { get; set; }

                private void OnBeforeInit(object sender, EventArgs e)
                {
                    this.EnvironmentMock
                        .SetupGet(x => x.Solution)
                        .Returns(this.SolutionMock.Object);

                    this.SolutionMock
                        .SetupGet(x => x.Projects)
                        .Returns(this.ProjectsFake);

                    this.ProjectsFake.AddItem(this.ProjectMock.Object);

                    this.ProjectMock
                        .SetupGet(x => x.FullName)
                        .Returns(@"project path\project.csproj");

                    this.ProjectMock
                        .SetupGet(x => x.UniqueName)
                        .Returns("Unique Name");

                    this.ProjectMock
                        .SetupGet(x => x.ProjectItems)
                        .Returns(this.ProjectItemsFake);

                    this.ProjectItemsFake.ContainingProject = this.ProjectMock.Object;
                    this.ProjectItemsFake.AddProjectItem(@"project path\test1.js");
                    this.ProjectItemsFake.AddProjectItem(@"project path\test2.txt");
                    this.ProjectItemsFake.AddProjectItem(@"project path\test3.ts");
                    this.ProjectItemsFake.AddProjectItem(@"project path\test4.json");

                    this.VisualStudioJSLintProviderMock
                        .Setup(x => x.LintProjectItems(It.IsAny<IList<ProjectItem>>(), this.Settings))
                        .Callback((IList<ProjectItem> x, JSLintNetSettings y) => this.LastItemsLinted = x)
                        .Returns(() => this.ErrorsFound);

                    this.VisualStudioJSLintProviderMock
                        .Setup(x => x.LoadSettings(this.ProjectMock.Object))
                        .Returns(this.Settings);
                }
            }
        }

        private abstract class BuildEventControllerTestableBase : EventControllerTestableBase<BuildEventController>
        {
            public BuildEventControllerTestableBase()
            {
                this.BuildEventsMock = new Mock<BuildEvents>();

                this.BeforeInit += this.OnBeforeInit;
            }

            public Mock<BuildEvents> BuildEventsMock { get; set; }

            private void OnBeforeInit(object sender, System.EventArgs e)
            {
                this.EventsMock
                    .SetupGet(x => x.BuildEvents)
                    .Returns(this.BuildEventsMock.Object);
            }
        }
    }
}
