﻿namespace JSLintNet.VisualStudio.Specifications.EventControllers
{
    using System;
    using System.Collections.Generic;
    using System.ComponentModel.Design;
    using System.IO;
    using EnvDTE;
    using EnvDTE80;
    using JSLintNet.QualityTools;
    using JSLintNet.QualityTools.Helpers;
    using JSLintNet.VisualStudio.EventControllers;
    using JSLintNet.VisualStudio.Specifications.Fakes;
    using Microsoft.VisualStudio.Shell;
    using Moq;
    using Xunit;

    public class MenuEventControllerUnit
    {
        public class OnBeforeErrorListClear : UnitBase
        {
            [Fact]
            public void Should_set_to_hidden_when_no_errors_exist()
            {
                using (var testable = new OnBeforeErrorListClearTestable())
                {
                    testable.Init();
                    testable.BeforeHandler(testable.MenuCommand, null);

                    testable.ErrorListProviderMock.VerifyGet(x => x.ErrorCount);
                    Assert.False(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_set_to_visible_when_errors_exist()
            {
                using (var testable = new OnBeforeErrorListClearTestable())
                {
                    testable.ErrorListProviderMock
                        .SetupGet(x => x.ErrorCount)
                        .Returns(10);

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    testable.ErrorListProviderMock.VerifyGet(x => x.ErrorCount);
                    Assert.True(testable.MenuCommand.Visible);
                }
            }

            private class OnBeforeErrorListClearTestable : MenuEventControllerTestableBase
            {
                public override CommandID CommandID
                {
                    get
                    {
                        return JSLintCommands.ErrorListClear;
                    }
                }
            }
        }

        public class OnErrorListClear : UnitBase
        {
            [Fact]
            public void Should_clear_all_tasks()
            {
                using (var testable = new OnErrorListClearTestable())
                {
                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    testable.ErrorListProviderMock.Verify(x => x.ClearAllErrors());
                }
            }

            private class OnErrorListClearTestable : MenuEventControllerTestableBase
            {
                public override CommandID CommandID
                {
                    get
                    {
                        return JSLintCommands.ErrorListClear;
                    }
                }
            }
        }

        public class OnBeforeItemNodeRun : UnitBase
        {
            [Fact]
            public void Should_set_to_hidden_when_selection_contains_any_unlintables()
            {
                using (var testable = new OnBeforeItemNodeRunTestable())
                {
                    testable.AddSelectedItem("lintable.js");
                    testable.AddSelectedItem("notlintable.txt");
                    testable.AddSelectedItem("lintable.json");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.False(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_set_to_hidden_when_selection_is_empty()
            {
                using (var testable = new OnBeforeItemNodeRunTestable())
                {
                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.False(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_set_to_visible_when_selection_only_contains_lintables()
            {
                using (var testable = new OnBeforeItemNodeRunTestable())
                {
                    testable.AddSelectedItem("lintable.js");
                    testable.AddSelectedItem("lintable.json");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.True(testable.MenuCommand.Visible);
                }
            }

            private class OnBeforeItemNodeRunTestable : ItemNodeTestableBase
            {
                public override CommandID CommandID
                {
                    get
                    {
                        return JSLintCommands.ItemNodeRun;
                    }
                }
            }
        }

        public class OnItemNodeRun : UnitBase
        {
            [Fact]
            public void Should_process_all_selected_lintables()
            {
                using (var testable = new OnItemNodeRunTestable())
                {
                    testable.AddSelectedItem("lintable.js");
                    testable.AddSelectedItem("notlintable.txt");
                    testable.AddSelectedItem("lintable.json");

                    IList<ProjectItem> projectItems = null;

                    testable.VisualStudioJSLintProviderMock
                        .Setup(x => x.LintProjectItems(It.IsAny<IList<ProjectItem>>()))
                        .Callback((IList<ProjectItem> x) => projectItems = x);

                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    Assert.NotNull(projectItems);
                    Assert.Equal(2, projectItems.Count);
                }
            }

            private class OnItemNodeRunTestable : ItemNodeTestableBase
            {
                public override CommandID CommandID
                {
                    get
                    {
                        return JSLintCommands.ItemNodeRun;
                    }
                }
            }
        }

        public class OnBeforeCodeWindowRun : UnitBase
        {
            [Fact]
            public void Should_set_to_hidden_when_active_document_is_unlintable()
            {
                using (var testable = new OnBeforeCodeWindowRunTestable())
                {
                    testable.DocumentName = "unlintable.txt";

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    testable.EnvironmentMock.VerifyGet(x => x.ActiveDocument);
                    Assert.False(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_set_to_visible_when_active_document_is_lintable()
            {
                using (var testable = new OnBeforeCodeWindowRunTestable())
                {
                    testable.DocumentName = "lintable.json";

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    testable.EnvironmentMock.VerifyGet(x => x.ActiveDocument);
                    Assert.True(testable.MenuCommand.Visible);
                }
            }

            private class OnBeforeCodeWindowRunTestable : CodeWindowRunTestableBase
            {
            }
        }

        public class OnCodeWindowRun : UnitBase
        {
            [Fact]
            public void Should_not_process_active_document_if_unlintable()
            {
                using (var testable = new OnCodeWindowRunTestable())
                {
                    testable.DocumentName = "script.coffee";

                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    testable.EnvironmentMock.VerifyGet(x => x.ActiveDocument);
                    testable.VisualStudioJSLintProviderMock.Verify(x => x.LintDocument(testable.DocumentMock.Object), Times.Never());
                }
            }

            [Fact]
            public void Should_process_active_document_if_lintable()
            {
                using (var testable = new OnCodeWindowRunTestable())
                {
                    testable.DocumentName = "script.js";

                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    testable.EnvironmentMock.VerifyGet(x => x.ActiveDocument);
                    testable.VisualStudioJSLintProviderMock.Verify(x => x.LintDocument(testable.DocumentMock.Object));
                }
            }

            private class OnCodeWindowRunTestable : CodeWindowRunTestableBase
            {
            }
        }

        public class OnBeforeItemNodeIgnore : UnitBase
        {
            [Fact]
            public void Should_hide_with_multiple_selections()
            {
                using (var testable = new OnBeforeItemNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\file.js");
                    testable.AddSelectedItem(@"data.json");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.False(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_hide_with_no_selections()
            {
                using (var testable = new OnBeforeItemNodeIgnoreTestable())
                {
                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.False(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_hide_with_one_non_lintable()
            {
                using (var testable = new OnBeforeItemNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\non.lintable");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.False(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_show_with_one_lintable()
            {
                using (var testable = new OnBeforeItemNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\file.js");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.True(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_check_and_disable_when_lintable_is_implicitly_ignored()
            {
                using (var testable = new OnBeforeItemNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\file.js");
                    testable.Settings.Ignore.Add(@"some");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.False(testable.MenuCommand.Enabled);
                    Assert.True(testable.MenuCommand.Checked);
                }
            }

            [Fact]
            public void Should_check_and_enable_when_lintable_is_explictly_ignored()
            {
                using (var testable = new OnBeforeItemNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\file.js");
                    testable.Settings.Ignore.Add(@"some\path\to\file.js");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.True(testable.MenuCommand.Enabled);
                    Assert.True(testable.MenuCommand.Checked);
                }
            }

            [Fact]
            public void Should_enable_but_not_check_when_lintable_is_not_ignored()
            {
                using (var testable = new OnBeforeItemNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\file.js");
                    testable.Settings.Ignore.Add(@"other\path");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.True(testable.MenuCommand.Enabled);
                    Assert.False(testable.MenuCommand.Checked);
                }
            }

            private class OnBeforeItemNodeIgnoreTestable : ItemNodeTestableBase
            {
                public override CommandID CommandID
                {
                    get
                    {
                        return JSLintCommands.ItemNodeIgnore;
                    }
                }
            }
        }

        public class OnItemNodeIgnore : UnitBase
        {
            [Fact]
            public void Should_remove_same_path_with_different_case()
            {
                using (var testable = new OnItemNodeIgnoreTestable())
                {
                    testable.Settings.Ignore.Add(@"SOME\PATH\TO\FILE.JS");

                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    Assert.DoesNotContain(@"SOME\PATH\TO\FILE.JS", testable.Settings.Ignore);
                }
            }

            [Fact]
            public void Should_remove_same_path_with_different_separators()
            {
                using (var testable = new OnItemNodeIgnoreTestable())
                {
                    testable.Settings.Ignore.Add(@"/SOME/PATH/TO/FILE.JS");

                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    Assert.DoesNotContain(@"/SOME/PATH/TO/FILE.JS", testable.Settings.Ignore);
                }
            }

            [Fact]
            public void Should_add_new_ignore_path()
            {
                using (var testable = new OnItemNodeIgnoreTestable())
                {
                    testable.Settings.Ignore.Add(@"/SOME/PATH/");

                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    Assert.Contains(@"\some\path\to\file.js", testable.Settings.Ignore);
                }
            }

            [Fact]
            public void Should_always_save_changes()
            {
                using (var testable = new OnItemNodeIgnoreTestable())
                {
                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    testable.VisualStudioJSLintProviderMock.Verify(x => x.SaveSettings(testable.ProjectMock.Object, testable.Settings));
                }
            }

            private class OnItemNodeIgnoreTestable : ItemNodeTestableBase
            {
                public OnItemNodeIgnoreTestable()
                {
                    this.BeforeInit += this.OnBeforeInit;
                }

                public override CommandID CommandID
                {
                    get
                    {
                        return JSLintCommands.ItemNodeIgnore;
                    }
                }

                private void OnBeforeInit(object sender, EventArgs e)
                {
                    this.AddSelectedItem(@"some\path\to\file.js");
                }
            }
        }

        public class OnBeforeFolderNodeIgnore : UnitBase
        {
            [Fact]
            public void Should_hide_with_multiple_selections()
            {
                using (var testable = new OnBeforeFolderNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\");
                    testable.AddSelectedItem(@"data\");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.False(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_hide_with_no_selections()
            {
                using (var testable = new OnBeforeFolderNodeIgnoreTestable())
                {
                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.False(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_show_with_one_selection()
            {
                using (var testable = new OnBeforeFolderNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.True(testable.MenuCommand.Visible);
                }
            }

            [Fact]
            public void Should_check_and_disable_when_folder_is_implicitly_ignored()
            {
                using (var testable = new OnBeforeFolderNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\");
                    testable.Settings.Ignore.Add(@"some");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.False(testable.MenuCommand.Enabled);
                    Assert.True(testable.MenuCommand.Checked);
                }
            }

            [Fact]
            public void Should_check_and_enable_when_folder_is_explictly_ignored()
            {
                using (var testable = new OnBeforeFolderNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\");
                    testable.Settings.Ignore.Add(@"some\path\to\");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.True(testable.MenuCommand.Enabled);
                    Assert.True(testable.MenuCommand.Checked);
                }
            }

            [Fact]
            public void Should_enable_but_not_check_when_folder_is_not_ignored()
            {
                using (var testable = new OnBeforeFolderNodeIgnoreTestable())
                {
                    testable.AddSelectedItem(@"some\path\to\");
                    testable.Settings.Ignore.Add(@"other\path");

                    testable.Init();
                    testable.BeforeHandler.Invoke(testable.MenuCommand, null);

                    Assert.True(testable.MenuCommand.Enabled);
                    Assert.False(testable.MenuCommand.Checked);
                }
            }

            private class OnBeforeFolderNodeIgnoreTestable : ItemNodeTestableBase
            {
                public override CommandID CommandID
                {
                    get
                    {
                        return JSLintCommands.FolderNodeIgnore;
                    }
                }
            }
        }

        public class OnFolderNodeIgnore : UnitBase
        {
            [Fact]
            public void Should_remove_same_path_with_different_case()
            {
                using (var testable = new OnFolderNodeIgnoreTestable())
                {
                    testable.Settings.Ignore.Add(@"SOME\PATH\TO\");

                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    Assert.DoesNotContain(@"SOME\PATH\TO\", testable.Settings.Ignore);
                }
            }

            [Fact]
            public void Should_remove_same_path_with_different_separators()
            {
                using (var testable = new OnFolderNodeIgnoreTestable())
                {
                    testable.Settings.Ignore.Add(@"/SOME/PATH/TO/");

                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    Assert.DoesNotContain(@"/SOME/PATH/TO/", testable.Settings.Ignore);
                }
            }

            [Fact]
            public void Should_add_new_ignore_path()
            {
                using (var testable = new OnFolderNodeIgnoreTestable())
                {
                    testable.Settings.Ignore.Add(@"/SOME/PATH/");

                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    Assert.Contains(@"\some\path\to\", testable.Settings.Ignore);
                }
            }

            [Fact]
            public void Should_always_save_changes()
            {
                using (var testable = new OnFolderNodeIgnoreTestable())
                {
                    testable.Init();
                    testable.MenuCommand.Invoke(testable.MenuCommand);

                    testable.VisualStudioJSLintProviderMock.Verify(x => x.SaveSettings(testable.ProjectMock.Object, testable.Settings));
                }
            }

            private class OnFolderNodeIgnoreTestable : ItemNodeTestableBase
            {
                public OnFolderNodeIgnoreTestable()
                {
                    this.BeforeInit += this.OnBeforeInit;
                }

                public override CommandID CommandID
                {
                    get
                    {
                        return JSLintCommands.FolderNodeIgnore;
                    }
                }

                private void OnBeforeInit(object sender, EventArgs e)
                {
                    this.AddSelectedItem(@"some\path\to\");
                }
            }
        }

        private abstract class MenuEventControllerTestableBase : EventControllerTestableBase<MenuEventController>
        {
            public MenuEventControllerTestableBase()
            {
                this.MenuCommandServiceMock = new Mock<IMenuCommandService>();

                this.BeforeInit += this.OnBeforeInit;

                this.AfterInit += this.OnAfterInit;
            }

            public Mock<IMenuCommandService> MenuCommandServiceMock { get; set; }

            public abstract CommandID CommandID { get; }

            public OleMenuCommand MenuCommand { get; set; }

            public EventHandler BeforeHandler { get; set; }

            private void OnBeforeInit(object sender, EventArgs e)
            {
                this.ServiceProviderMock
                    .Setup(x => x.GetService(typeof(IMenuCommandService)))
                    .Returns(this.MenuCommandServiceMock.Object);

                this.MenuCommandServiceMock
                    .Setup(x => x.AddCommand(It.Is<MenuCommand>(y => y.CommandID == this.CommandID)))
                    .Callback((MenuCommand x) => this.MenuCommand = (OleMenuCommand)x);
            }

            private void OnAfterInit(object sender, EventArgs e)
            {
                this.BeforeHandler = ReflectionHelper.GetFieldValue<EventHandler>(this.MenuCommand, "beforeQueryStatusHandler");
            }
        }

        private abstract class ItemNodeTestableBase : MenuEventControllerTestableBase
        {
            private List<UIHierarchyItem> selectedItems = new List<UIHierarchyItem>();

            public ItemNodeTestableBase()
            {
                this.ProjectMock = new Mock<Project>();
                this.ProjectFullName = @"c:\full\path\to\project.csproj";
                this.Settings = new JSLintNetSettings();

                this.BeforeInit += this.OnBeforeInit;
            }

            internal JSLintNetSettings Settings { get; set; }

            internal Mock<Project> ProjectMock { get; set; }

            protected string ProjectFullName { get; set; }

            public Mock<UIHierarchyItem> AddSelectedItem(string fileName, bool isLink = false)
            {
                var hierarchyItemMock = new Mock<UIHierarchyItem>();
                var projectItemMock = new Mock<ProjectItem>();
                var name = Path.GetFileName(fileName);
                var propertiesFake = new PropertiesFake();
                var path = Path.GetDirectoryName(this.ProjectFullName);

                propertiesFake.AddProperty("IsLink", isLink);

                projectItemMock
                    .Setup(x => x.get_FileNames(0))
                    .Returns(Path.Combine(path, fileName));

                projectItemMock
                    .SetupGet(x => x.Name)
                    .Returns(name);

                projectItemMock
                    .SetupGet(x => x.Properties)
                    .Returns(propertiesFake);

                projectItemMock
                    .SetupGet(x => x.ContainingProject)
                    .Returns(this.ProjectMock.Object);

                hierarchyItemMock
                    .SetupGet(x => x.Name)
                    .Returns(name);

                hierarchyItemMock
                    .SetupGet(x => x.Object)
                    .Returns(projectItemMock.Object);

                this.selectedItems.Add(hierarchyItemMock.Object);

                return hierarchyItemMock;
            }

            private void OnBeforeInit(object sender, EventArgs e)
            {
                var solutionExplorerWindowMock = new Mock<UIHierarchy>();
                var toolWindowsMock = new Mock<ToolWindows>();

                this.ProjectMock
                    .SetupGet(x => x.FullName)
                    .Returns(() => this.ProjectFullName);

                solutionExplorerWindowMock
                    .SetupGet(x => x.SelectedItems)
                    .Returns(() => this.selectedItems.ToArray());

                toolWindowsMock
                    .Setup(x => x.GetToolWindow(Constants.vsWindowKindSolutionExplorer))
                    .Returns(solutionExplorerWindowMock.Object);

                this.EnvironmentMock
                    .SetupGet(x => x.ToolWindows)
                    .Returns(toolWindowsMock.Object);

                this.VisualStudioJSLintProviderMock
                    .Setup(x => x.LoadSettings(It.IsAny<Project>()))
                    .Returns(this.Settings);
            }
        }

        private abstract class CodeWindowRunTestableBase : MenuEventControllerTestableBase
        {
            public CodeWindowRunTestableBase()
            {
                this.DocumentMock = new Mock<Document>();

                this.BeforeInit += this.OnBeforeInit;
            }

            public Mock<Document> DocumentMock { get; set; }

            public string DocumentName { get; set; }

            public override CommandID CommandID
            {
                get
                {
                    return JSLintCommands.CodeWindowRun;
                }
            }

            private void OnBeforeInit(object sender, EventArgs e)
            {
                this.DocumentMock
                    .SetupGet(x => x.Name)
                    .Returns(() => this.DocumentName);

                this.EnvironmentMock
                    .SetupGet(x => x.ActiveDocument)
                    .Returns(this.DocumentMock.Object);
            }
        }
    }
}
