﻿namespace JSLintNet.VisualStudio.Specifications
{
    using System;
    using System.Collections.Generic;
    using System.IO;
    using System.Text;
    using EnvDTE;
    using JSLintNet.Abstractions;
    using JSLintNet.Json;
    using JSLintNet.Properties;
    using JSLintNet.QualityTools;
    using JSLintNet.QualityTools.Fakes;
    using JSLintNet.VisualStudio.Errors;
    using JSLintNet.VisualStudio.Specifications.Fakes;
    using Microsoft.VisualStudio.Shell.Interop;
    using Moq;
    using Xunit;

    public class VisualStudioJSLintProviderUnit
    {
        public class GetSettings : UnitBase
        {
            [Fact]
            public void Should_get_linked_settings_file_from_project_if_it_exists()
            {
                using (var testable = new GetSettingsTestable())
                {
                    var settingsPath = Path.Combine(@"some\path", JSLintNetSettings.FileName);

                    testable.ProjectItemsFake.AddProjectItem(settingsPath, true);
                    testable.FileSystemWrapperMock
                        .Setup(y => y.FileExists(settingsPath))
                        .Returns(true);

                    testable.Instance.LoadSettings(testable.ProjectMock.Object);

                    testable.FileSystemWrapperMock.Verify(x => x.ReadAllText(settingsPath, Encoding.UTF8));
                }
            }

            [Fact]
            public void Should_get_settings_from_project_root_if_it_exists()
            {
                using (var testable = new GetSettingsTestable())
                {
                    testable.FileSystemWrapperMock
                        .Setup(x => x.FileExists(@"solution\project\JSLintNet.json"))
                        .Returns(true);

                    testable.Instance.LoadSettings(testable.ProjectMock.Object);

                    testable.FileSystemWrapperMock.Verify(x => x.ReadAllText(@"solution\project\JSLintNet.json", Encoding.UTF8));
                }
            }

            [Fact]
            public void Should_deserialize_settings_file_if_it_exists()
            {
                using (var testable = new GetSettingsTestable())
                {
                    testable.FileSystemWrapperMock
                        .Setup(x => x.FileExists(It.IsAny<string>()))
                        .Returns(true);

                    testable.FileSystemWrapperMock
                        .Setup(x => x.ReadAllText(It.IsAny<string>(), Encoding.UTF8))
                        .Returns("JSON SETTINGS");

                    testable.Instance.LoadSettings(testable.ProjectMock.Object);

                    testable.JsonProviderMock.Verify(x => x.DeserializeSettings("JSON SETTINGS"));
                }
            }

            private class GetSettingsTestable : VisualStudioJSLintProviderTestableBase
            {
                public GetSettingsTestable()
                {
                    this.ProjectItemsFake = new ProjectItemsFake();

                    this.BeforeInit += this.OnBeforeInit;
                }

                public ProjectItemsFake ProjectItemsFake { get; set; }

                private void OnBeforeInit(object sender, EventArgs e)
                {
                    this.ProjectMock
                        .SetupGet(x => x.ProjectItems)
                        .Returns(this.ProjectItemsFake);
                }
            }
        }

        public class LintProjectItems : UnitBase
        {
            [Fact]
            public void Should_stop_processing_when_error_limit_reached()
            {
                using (var testable = new LintProjectItemsTestable())
                {
                    testable.Settings.ErrorLimit = 10;

                    for (int i = 1; i <= 10; i++)
                    {
                        testable.AddFile("file" + i + ".js", 2);
                    }

                    testable.Instance.LintProjectItems(testable.ProjectItems, testable.Settings);

                    testable.ErrorListProviderMock.Verify(x => x.AddCustomError(Resources.ErrorLimitReachedFormat, 10));
                    testable.JSLintContextMock.Verify(x => x.Lint("file5.js contents", null));
                    testable.JSLintContextMock.Verify(x => x.Lint("file6.js contents", null), Times.Never());
                }
            }

            [Fact]
            public void Should_stop_processing_when_file_limit_reached()
            {
                using (var testable = new LintProjectItemsTestable())
                {
                    testable.Settings.FileLimit = 5;

                    for (int i = 1; i <= 10; i++)
                    {
                        testable.AddFile("file" + i + ".js", 0);
                    }

                    testable.Instance.LintProjectItems(testable.ProjectItems, testable.Settings);

                    testable.ErrorListProviderMock.Verify(x => x.AddCustomError(Resources.FileLimitReachedFormat, 5));
                    testable.JSLintContextMock.Verify(x => x.Lint("file5.js contents", null));
                    testable.JSLintContextMock.Verify(x => x.Lint("file6.js contents", null), Times.Never());
                }
            }

            [Fact]
            public void Should_stop_processing_when_exception_limit_reached()
            {
                using (var testable = new LintProjectItemsTestable())
                {
                    testable.JSLintContextMock
                        .Setup(x => x.Lint(It.IsAny<string>(), It.IsAny<JSLintOptions>()))
                        .Throws<Exception>();

                    var document = new Mock<Document>();

                    document
                        .Setup(x => x.Object(It.IsAny<string>()))
                        .Throws<Exception>();

                    for (int i = 1; i <= 51; i++)
                    {
                        var mockItem = testable.AddFile("file" + i + ".js", 0, true);

                        mockItem
                            .SetupGet(x => x.Document)
                            .Returns(document.Object);
                    }

                    testable.Instance.LintProjectItems(testable.ProjectItems, testable.Settings);

                    testable.ErrorListProviderMock.Verify(x => x.AddCustomError(Resources.ExceptionLimitReachedFormat, 50));

                    Mock.Get(testable.ProjectItems[49]).Verify(x => x.Document);
                    Mock.Get(testable.ProjectItems[50]).Verify(x => x.Document, Times.Never());
                }
            }

            private class LintProjectItemsTestable : VisualStudioJSLintProviderTestableBase
            {
                public LintProjectItemsTestable()
                {
                    this.ErrorListProviderMock = this.AutoMocker.Mock<IJSLintErrorListProvider>();
                    this.ServiceProviderMock = this.AutoMocker.Mock<IServiceProvider>();
                    this.JSLintFactoryMock = this.AutoMocker.Mock<IJSLintFactory>();
                    this.JSLintContextMock = new Mock<IJSLintContext>();
                    this.Settings = new JSLintNetSettings();
                    this.ProjectItems = new List<ProjectItem>();

                    this.BeforeInit += this.OnBeforeInit;
                }

                public Mock<IJSLintErrorListProvider> ErrorListProviderMock { get; set; }

                public Mock<IServiceProvider> ServiceProviderMock { get; set; }

                public Mock<IJSLintFactory> JSLintFactoryMock { get; set; }

                public Mock<IJSLintContext> JSLintContextMock { get; set; }

                public JSLintNetSettings Settings { get; set; }

                public IList<ProjectItem> ProjectItems { get; set; }

                public Mock<ProjectItem> AddFile(string fileName, int errorCount = 0, bool noMocks = false)
                {
                    var itemMock = new Mock<ProjectItem>();

                    itemMock
                        .SetupGet(x => x.ContainingProject)
                        .Returns(this.ProjectMock.Object);

                    itemMock
                        .Setup(x => x.get_FileNames(0))
                        .Returns(fileName);

                    if (!noMocks)
                    {
                        var source = fileName + " contents";

                        this.FileSystemWrapperMock
                            .Setup(x => x.ReadAllText(fileName, Encoding.UTF8))
                            .Returns(source);

                        this.JSLintContextMock
                            .Setup(x => x.Lint(source, null))
                            .Returns(new JSLintDataFake(fileName, errorCount));
                    }

                    this.ProjectItems.Add(itemMock.Object);

                    return itemMock;
                }

                private void OnBeforeInit(object sender, EventArgs e)
                {
                    this.JSLintFactoryMock
                        .Setup(x => x.CreateContext())
                        .Returns(this.JSLintContextMock.Object);

                    var solutionServiceMock = new Mock<IVsSolution>();
                    var statusBarMock = new Mock<IVsStatusbar>();

                    this.ServiceProviderMock
                        .Setup(x => x.GetService(typeof(SVsSolution)))
                        .Returns(solutionServiceMock.Object);

                    this.ServiceProviderMock
                        .Setup(x => x.GetService(typeof(SVsStatusbar)))
                        .Returns(statusBarMock.Object);
                }
            }
        }

        private abstract class VisualStudioJSLintProviderTestableBase : TestableBase<VisualStudioJSLintProvider>
        {
            public VisualStudioJSLintProviderTestableBase()
            {
                this.FileSystemWrapperMock = this.AutoMocker.Mock<IFileSystemWrapper>();
                this.JsonProviderMock = this.AutoMocker.Mock<IJsonProvider>();
                this.ProjectMock = new Mock<Project>();

                this.BeforeInit += this.OnBeforeInit;
            }

            public Mock<IFileSystemWrapper> FileSystemWrapperMock { get; private set; }

            public Mock<IJsonProvider> JsonProviderMock { get; private set; }

            public Mock<Project> ProjectMock { get; set; }

            private void OnBeforeInit(object sender, EventArgs e)
            {
                this.ProjectMock
                    .SetupGet(x => x.FullName)
                    .Returns(@"solution\project\project.csproj");

                this.ProjectMock
                    .SetupGet(x => x.UniqueName)
                    .Returns(@"project.csproj");
            }
        }
    }
}
